﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using WiimoteDevice;
using Setting;

namespace WiiPC
{
    public partial class MainWindow : Form, IControllerChanger
    {
        private WiimoteDevice.Controller Wiimote = new WiimoteDevice.Controller();
        private IWiiController WorkingController;
        private int currentSetting = 1;
        private Popup.PopupMessage PopupMessage = new Popup.PopupMessage();

        private delegate void LabelTextDelegate(Label label, String text);
        private delegate void ToolStripProgressDelegate(ToolStripProgressBar bar, int value);

        private delegate void PopupDelegate(String text);

        private void SetLabelText(Label label, String text) {
            label.Text = text;
        }

        private void SetProgressValue(ToolStripProgressBar bar, int value) {
            bar.Value = value;
        }

        public MainWindow()
        {
            InitializeComponent();
            this.Wiimote.ButtonPress += new EventHandler<ButtonPressEventArgs>(Wiimote_ButtonPress);
            this.Wiimote.ButtonUp += new EventHandler<ButtonUpEventArgs>(Wiimote_ButtonUp);

            this.wiiController1.ControllerChanger = this;
            this.wiiController2.ControllerChanger = this;
            this.wiiController3.ControllerChanger = this;
            this.wiiController4.ControllerChanger = this;
        }

        protected override void OnLoad(EventArgs e) {
            base.OnLoad(e);
            this.labelStatus.Text = "Disconnected";
            ChangeController(1);
            this.LoadSetting(Setting.Loader.Load());
        }

        protected override void OnClosing(CancelEventArgs e) {
            this.Wiimote.Disconnect();
            this.progressBattery.Value = 0;
            base.OnClosing(e);
        }

        void Wiimote_ButtonPress(object sender, ButtonPressEventArgs e) {
            this.BeginInvoke(new ToolStripProgressDelegate(SetProgressValue), progressBattery, (int)(this.Wiimote.Battery));
            WorkingController.ButtonPressHandler(e);
        }

        void Wiimote_ButtonUp(object sender, ButtonUpEventArgs e) {
            WorkingController.ButtonUpHandler(e);
        }


        private void ReflectLED() {
            if (this.displaySettingLightToolStripMenuItem.Checked == false) {
                Wiimote.SetLED(false, false, false, false);
            } else {
                Wiimote.SetLED(currentSetting == 1, currentSetting == 2, currentSetting == 3, currentSetting == 4);
            }
        }

        private void connectCToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try {
                this.Wiimote.Connect();
                this.Wiimote.SetRumble(500);
                this.connectCToolStripMenuItem.Enabled = false;
                this.disconnectToolStripMenuItem.Enabled = true;
                this.labelStatus.Text = "Connected";
                this.progressBattery.Value = (int)(this.Wiimote.Battery);

                if (this.displaySettingLightToolStripMenuItem.Checked == true) {
                    Wiimote.SetLED(currentSetting == 1, currentSetting == 2, currentSetting == 3, currentSetting == 4);
                }

            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "Connect Fail", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void disconnectToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Wiimote.Disconnect();
            this.progressBattery.Value = 0;
            this.connectCToolStripMenuItem.Enabled = true;
            this.disconnectToolStripMenuItem.Enabled = false;
            this.labelStatus.Text = "Disconnected";
        }

        private void 끝내기XToolStripMenuItem_Click(object sender, EventArgs e) {
            this.Close();
        }

        public void ChangeController(int controllerNum) {
            this.currentSetting = controllerNum;
            switch (controllerNum) {
                case 1: this.WorkingController = wiiController1;
                    this.Invoke(new LabelTextDelegate(SetLabelText), labelSetting, "Setting 1"); 
                    if(this.displaySettingChangedDToolStripMenuItem.Checked == true) {
                        ShowPopup("Setting 1");
                    }
                    break;
                case 2: this.WorkingController = wiiController2; 
                    this.Invoke(new LabelTextDelegate(SetLabelText), labelSetting, "Setting 2"); 
                    if(this.displaySettingChangedDToolStripMenuItem.Checked == true) {
                        ShowPopup("Setting 2");
                    }
                    break;
                case 3: this.WorkingController = wiiController3; 
                    this.Invoke(new LabelTextDelegate(SetLabelText), labelSetting, "Setting 3");
                    if(this.displaySettingChangedDToolStripMenuItem.Checked == true) {
                        ShowPopup("Setting 3");
                    }
                    break;
                case 4: this.WorkingController = wiiController4; 
                    this.Invoke(new LabelTextDelegate(SetLabelText), labelSetting, "Setting 4"); 
                    if(this.displaySettingChangedDToolStripMenuItem.Checked == true) {
                        ShowPopup("Setting 4");
                    }
                    break;
            }

            ReflectLED();
        }

        private void displaySettingChangedDToolStripMenuItem_Click(object sender, EventArgs e) {
            // 미구현
        }

        private void displaySettingLightToolStripMenuItem_CheckedChanged(object sender, EventArgs e) {
            ReflectLED();
        }

        private void displaySettingLightToolStripMenuItem_Click(object sender, EventArgs e) {
            this.displaySettingLightToolStripMenuItem.Checked = !this.displaySettingLightToolStripMenuItem.Checked;
        }

        private void exitXToolStripMenuItem_Click(object sender, EventArgs e) {
            this.Wiimote.Disconnect();
            this.Close();
        }

        private void notifyIcon1_MouseClick(object sender, MouseEventArgs e) {
            if (e.Button == System.Windows.Forms.MouseButtons.Left) {
                this.Visible = true;
                this.WindowState = FormWindowState.Normal;
                this.TopMost = true;
                this.TopMost = false;
            }
        }

        private void displaySettingChangedDToolStripMenuItem_Click_1(object sender, EventArgs e) {
            this.displaySettingChangedDToolStripMenuItem.Checked = !this.displaySettingChangedDToolStripMenuItem.Checked;
        }

        private void saveAllSettingSToolStripMenuItem_Click(object sender, EventArgs e) {
            if (Setting.Saver.Save(ToWiimoteSetting()) == true) {
                MessageBox.Show("Success", "Save Settings", MessageBoxButtons.OK, MessageBoxIcon.Information);
            } else {
                MessageBox.Show("Fail", "Save Settings", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ShowPopup(String msg) {
            this.BeginInvoke(new PopupDelegate(PopupMessage.Show), msg);
        }

        public WiimoteSetting ToWiimoteSetting() {
            WiimoteSetting setting = new WiimoteSetting(
                this.displaySettingChangedDToolStripMenuItem.Checked,
                this.displaySettingLightToolStripMenuItem.Checked,
                wiiController1.ToControllerSetting(1),
                wiiController2.ToControllerSetting(2),
                wiiController3.ToControllerSetting(3),
                wiiController4.ToControllerSetting(4)
                );
            return setting;
        }


        public void LoadSetting(WiimoteSetting setting) {
            try {
                this.displaySettingChangedDToolStripMenuItem.Checked = setting.DisplaySettingChanging;
                this.displaySettingLightToolStripMenuItem.Checked = setting.DisplayLED;
                wiiController1.LoadSetting(setting.Controller[0]);
                wiiController2.LoadSetting(setting.Controller[1]);
                wiiController3.LoadSetting(setting.Controller[2]);
                wiiController4.LoadSetting(setting.Controller[3]);
            } catch (Exception) {
                MessageBox.Show("Fail to load setting.", "Load setting", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void helpToolStripMenuItem_Click(object sender, EventArgs e) {
            new AboutBox1().ShowDialog();
        }


        protected override void OnResize(EventArgs e) {
            base.OnResize(e);
            if (this.WindowState == FormWindowState.Minimized) {
                this.Visible = false;
            }
        }
    }
}
